
	.include    "jaguar.inc"
  .include    "graphics.inc"
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Begin STARTUP PICTURE CONFIGURATION -- Edit this to change startup picture
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PPP     	.equ    4      			; Pixels per Phrase (1-bit)
BMP_WIDTH   	.equ    256      		; Width in Pixels
BMP_HEIGHT  	.equ    232    			; Height in Pixels

BMP2_W	.equ	128
BMP2_H	.equ	163

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of STARTUP PICTURE CONFIGURATION
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Globals		
    .globl  _vidmem
		.globl  _bitmap
		.globl  _bitmap8
		.globl  _enemybitmap
		.globl  _anim
		.globl  _font813
		.globl  _font8x8_8b
		
		.globl _testsample
		.globl _testsampleend
		.globl _loopsample
		.globl _loopsampleend
		
		.globl _module
		
; Externals
		.extern	___main
    .extern gpucode
    .extern startgpucode
    .extern endgpucode 
    
BMP_PHRASES 	.equ    (BMP_WIDTH/PPP) 	; Width in Phrases
BMP_LINES   	.equ    (BMP_HEIGHT*2)  	; Height in Half Scanlines

LISTSIZE    	.equ    11       		; List length (in phrases)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Program Entry Point Follows...

		.text

    move.l  #INITSTACK,a7   	; Setup a stack
            	
    move.l  #BMP_WIDTH,d0
		move.l  #BMP_HEIGHT,d1	
    move.l  #CRY16,d2
    clr.l   d3	
		jsr   InitJaguar
	
  	jsr LoadGPUCode	
	
  	jsr 	InitLister     		; Initialize Object Display List
		move.l  #bitmaparray,_ActiveBitmapArray
	
	  move.w #0,CLUT
	  move.w #$FFFF,CLUT+2
	  lea  _bitmap8,a0
	  move.b #1,(a0)+
	  move.b #1,(a0)+
	  move.b #0,(a0)+
	  move.b #1,(a0)+
	  move.b #0,(a0)+
	  move.b #1,(a0)+
	  
    jmp 	___main			; Jump to main code


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; InitLister: Initialize Object List Processor List
;
;    Returns: Pre-word-swapped address of current object list in d0.l
;
;  Registers: d0.l/d1.l - Phrase being built
;             d2.l/d3.l - Branch and Link address overlays
;             d4.l      - Work register
;             a0.l      - Roving object list pointer
;             a1.l      - ptr to bitmap structure
		
InitLister:
		movem.l d1-d4/a0-a1,-(sp)		; Save registers
			
		lea     listbuf,a0

; Init display list
		move.l  #LISTSIZE*8,-(sp)
		move.l  a0,-(sp)
		jsr _InitDL
		move.l  (sp)+,a0
		addq.l  #4,sp		
		add.l d0,a0
	    
; Write a standard BITMAP object
    lea bitmaparray,a1
		move.l  #listbuf,d0
		swap    d0
    move.l  d0,CBitmapArrayDL(a1)    
    move.l  #4,CBitmapArrayCount(a1)
    move.l  #bitmapobjects,d0
    move.l  d0,CBitmapArrayPt(a1)
    move.l  d0,a1   
     
    move.l  #_vidmem,CBitmapAddr(a1)    
		clr.w   CBitmapX(a1)   
    clr.w   CBitmapY(a1)      
    move.w  #BMP_WIDTH,CBitmapW(a1)
    move.w  #BMP_HEIGHT,CBitmapH(a1)
    clr.w   CBitmapFlags(a1);
    
    move.l  a1,-(sp)
    move.l  a0,-(sp)
    jsr _AddBitmap 
    move.l  (sp)+,a0
    move.l  (sp)+,a1
    add.l   d0,a0
   lea CBitmapSz(a1),a1
   
; Write a second bitmap object    
    move.l  #_bitmap,CBitmapAddr(a1)
    move.w  #100,CBitmapX(a1)
    move.w  #100,CBitmapY(a1)    
    move.w  #BMP2_W,CBitmapW(a1)
    move.w  #BMP2_H,CBitmapH(a1)
    move.w  #O_TRANS,CBitmapFlags(a1)
    move.l  a1,-(sp)
    move.l  a0,-(sp)
    jsr _AddBitmap
    move.l  (sp)+,a0
    move.l  (sp)+,a1
    add.l d0,a0
    lea CBitmapSz(a1),a1
    
; Write a third bitmap object    
    move.l  #_anim,CBitmapAddr(a1)
    move.w  #150,CBitmapX(a1)
    move.w  #10,CBitmapY(a1)    
    move.w  #64,CBitmapW(a1)
    move.w  #64,CBitmapH(a1)
    move.w  #O_TRANS,CBitmapFlags(a1)
    move.l  a1,-(sp)
    move.l  a0,-(sp)
    jsr _AddBitmap
    move.l  (sp)+,a0
    move.l  (sp)+,a1
    add.l d0,a0    
    lea CBitmapSz(a1),a1
    
; Write a fourth bitmap object    
    move.l  #_bitmap8,CBitmapAddr(a1)
    move.w  #220,CBitmapX(a1)
    move.w  #0,CBitmapY(a1)    
    move.w  #64,CBitmapW(a1)
    move.w  #32,CBitmapH(a1)
    move.w  #0,CBitmapFlags(a1)
    move.w  #(1<<DEPTH8_FLAG),CBitmapCsFlags(a1)
    move.l  a1,-(sp)
    move.l  a0,-(sp)
    jsr _AddBitmap
    move.l  (sp)+,a0
    move.l  (sp)+,a1
    add.l d0,a0    
    lea CBitmapSz(a1),a1    
    
; Write a STOP object at end of list
    move.l  a0,-(sp)
    jsr _AddStopObject
    addq.l  #4,sp
      
		movem.l (sp)+,d1-d4/a0-a1
		rts


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Procedure: Load GPU code
;        Load GPU code into GPU RAM

LoadGPUCode:
    movem.l d0-d1/a0-a1,-(sp)
   	lea	gpucode,a0		; Get the address of the GPU code in ROM
		lea	startgpucode,a1	; Get destination address
		move.l	#endgpucode,d0		; and calculate length of GPU code
		sub.l	#startgpucode,d0
    asr.l #2,d0
    clr.l   d1    
    
    adda.l  #$8000,a1  ; GPU RAM 32 bit write access
    move.l  #PITCH1|PIXEL32|XADDPHR,A2_FLAGS
    move.l  a0,A2_BASE
    move.l  d1,A2_PIXEL     
    move.l  #PITCH1|PIXEL32|XADDPHR,A1_FLAGS
    move.l  a1,A1_BASE
    move.l  d1,A1_PIXEL
    or.l    #(1<<16),d0
    move.l  d0,B_COUNT
    move.l  #SRCEN|LFU_REPLACE,B_CMD
    movem.l (sp)+,d0-d1/a0-a1
    rts
 		


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		.data
		
		.phrase
_bitmap:	.incbin "data/test.cry"
_enemybitmap: .incbin "data/enemy.cry"
 	
    .phrase
_anim:  .incbin "data/anim.cry"
    
    .phrase
_font813: .incbin "font813.cry"
_font8x8_8b:  .incbin "font8x8_8.raw8"

    .dphrase
_testsample:  .incbin "data/ORCH.SAM"
_testsampleend:
_loopsample:  .incbin "data/WINDLP4_8.WAV"
_loopsampleend:

    .dphrase
_module:  .incbin "data/4aces-high2.mod"

		.bss
		.dphrase

_vidmem:
                .ds.w   76800
vidend:  
_bitmap8: ds.b  64*32

listbuf:    	.ds.l   LISTSIZE*2  		; Object List

bitmaparray:    .ds.l 1       ; count
            .ds.l 1       ; pointer to display list
            .ds.l 1       ; pointer to bitmap onjects
bitmapobjects:
            .ds.b CBitmapSz*4 ; bitmap objects

		.end

